import com.silanis.esl.sdk.external.signer.verification.domain.request.SignerVerificationRequest;
import com.silanis.esl.sdk.external.signer.verification.domain.response.SignerVerificationResponse;
import com.silanis.esl.sdk.external.signer.verification.domain.response.SignerVerificationResponsePayload;
import com.silanis.esl.sdk.external.signer.verification.domain.signer.SignerInfo;
import com.silanis.esl.sdk.external.signer.verification.domain.transaction.TransactionInfo;
import com.silanis.esl.sdk.external.signer.verification.processor.DefaultSignerVerificationProcessorFactory;
import com.silanis.esl.sdk.external.signer.verification.processor.SignerVerificationConfig;
import com.silanis.esl.sdk.external.signer.verification.processor.SignerVerificationProcessor;
import com.silanis.esl.sdk.external.signer.verification.processor.SignerVerificationProcessorFactory;
import com.silanis.esl.sdk.external.signer.verification.processor.jwt.HmacJwtSignerVerificationConfig;
import com.silanis.esl.sdk.external.signer.verification.url.SignerVerificationResponseUrlBuilder;

import java.net.URISyntaxException;
import java.util.UUID;

public class VerificationResponseTokenGeneration {
    private static final String SIGNING_SHARED_SECRET = "signing secret";
    private static final String ENCRYPTION_SHARED_SECRET = "encryption secret";
    private static final SignerVerificationProcessor signerVerificationProcessor;

    static {
        SignerVerificationConfig signerVerificationConfig = new HmacJwtSignerVerificationConfig(SIGNING_SHARED_SECRET, ENCRYPTION_SHARED_SECRET);
        SignerVerificationProcessorFactory signerVerificationProcessorFactory = new DefaultSignerVerificationProcessorFactory();
        signerVerificationProcessor = signerVerificationProcessorFactory.createSignerVerificationProcessor(signerVerificationConfig);
    }

    public static void main(String[] args) throws URISyntaxException {
        SignerVerificationRequest signerVerificationRequest = fetchSignerVerificationRequestFromSession();
        String documentIdentificationInfo = signerVerificationRequest.getDocumentIdentificationInfo();
        String callbackURL = signerVerificationRequest.getCallbackUrl();

        SignerVerificationResponse signerVerificationResponse = new SignerVerificationResponse()
                .setDocumentIdentificationInfo(documentIdentificationInfo)
                .setConfirmationPayload(UUID.randomUUID().toString())
                .setVerified(true);

        SignerVerificationResponsePayload signerVerificationResponsePayload = signerVerificationProcessor.toSignerVerificationResponsePayload(signerVerificationResponse);
        String redirectURL = new SignerVerificationResponseUrlBuilder()
                .withRootUrl(callbackURL)
                .withSignerVerificationResponsePayload(signerVerificationResponsePayload)
                .build();
        System.out.println("Redirecting signer to back to " + redirectURL);
    }

    /**
     For the actual implementation of the Verification Server, this should fetch the SignerVerificationRequest object from the user session.
     Here, we simply generate a sample SignerVerificationRequest so that we can show how to use it to generate a Signer Verification Response Token.
     */
    private static SignerVerificationRequest fetchSignerVerificationRequestFromSession() {
        return new SignerVerificationRequest()
                .setCallbackUrl("https://apps.esignlive.com/transaction/" + UUID.randomUUID().toString() + "/documents/" + UUID.randomUUID().toString())
                .setDocumentIdentificationInfo(UUID.randomUUID().toString())
                .setUserIdentityInfo(UUID.randomUUID().toString())
                .setTransactionInfo(new TransactionInfo()
                        .setTransactionName("Mortgage Application")
                        .setDocumentName("Mortgage Application Form"))
                .setSignerInfo(new SignerInfo()
                        .setSignerEmail("peter.young@yahoo.ca")
                        .setSignerFirstName("Peter")
                        .setSignerLastName("Young"));
    }


}
